import { findSourcePackage } from "./shared";
import fs, { constants as FS } from "fs/promises";
import { codeFrameColumns } from "next/dist/compiled/babel/code-frame";
import { launchEditor } from "../internal/helpers/launchEditor";
const currentSourcesByFile = new Map();
async function batchedTraceSource(project, frame) {
    const file = frame.file ? decodeURIComponent(frame.file) : undefined;
    if (!file) {
        return;
    }
    const sourceFrame = await project.traceSource(frame);
    if (!sourceFrame) {
        return;
    }
    let source;
    // Don't show code frames for node_modules. These can also often be large bundled files.
    if (!sourceFrame.file.includes("node_modules") && !sourceFrame.isInternal) {
        let sourcePromise = currentSourcesByFile.get(sourceFrame.file);
        if (!sourcePromise) {
            sourcePromise = project.getSourceForAsset(sourceFrame.file);
            currentSourcesByFile.set(sourceFrame.file, sourcePromise);
            setTimeout(()=>{
                // Cache file reads for 100ms, as frames will often reference the same
                // files and can be large.
                currentSourcesByFile.delete(sourceFrame.file);
            }, 100);
        }
        source = await sourcePromise;
    }
    var _sourceFrame_methodName, _ref;
    return {
        frame: {
            file: sourceFrame.file,
            lineNumber: sourceFrame.line,
            column: sourceFrame.column,
            methodName: (_ref = (_sourceFrame_methodName = sourceFrame.methodName) != null ? _sourceFrame_methodName : frame.methodName) != null ? _ref : "<unknown>",
            arguments: []
        },
        source: source != null ? source : null
    };
}
export async function createOriginalStackFrame(project, frame) {
    const traced = await batchedTraceSource(project, frame);
    if (!traced) {
        const sourcePackage = findSourcePackage(frame.file);
        if (sourcePackage) return {
            sourcePackage
        };
        return null;
    }
    var _traced_frame_lineNumber, _traced_frame_column;
    return {
        originalStackFrame: traced.frame,
        originalCodeFrame: traced.source === null ? null : codeFrameColumns(traced.source, {
            start: {
                // 1-based, but -1 means start line without highlighting
                line: (_traced_frame_lineNumber = traced.frame.lineNumber) != null ? _traced_frame_lineNumber : -1,
                // 1-based, but 0 means whole line without column highlighting
                column: (_traced_frame_column = traced.frame.column) != null ? _traced_frame_column : 0
            }
        }, {
            forceColor: true
        }),
        sourcePackage: findSourcePackage(traced.frame.file)
    };
}
export function getOverlayMiddleware(project) {
    return async function(req, res) {
        const { pathname, searchParams } = new URL(req.url, "http://n");
        var _searchParams_get, _searchParams_get1;
        const frame = {
            file: searchParams.get("file"),
            methodName: searchParams.get("methodName"),
            line: parseInt((_searchParams_get = searchParams.get("lineNumber")) != null ? _searchParams_get : "0", 10) || 0,
            column: parseInt((_searchParams_get1 = searchParams.get("column")) != null ? _searchParams_get1 : "0", 10) || 0,
            isServer: searchParams.get("isServer") === "true"
        };
        if (pathname === "/__nextjs_original-stack-frame") {
            let originalStackFrame;
            try {
                originalStackFrame = await createOriginalStackFrame(project, frame);
            } catch (e) {
                res.statusCode = 500;
                res.write(e.message);
                res.end();
                return;
            }
            if (originalStackFrame === null) {
                res.statusCode = 404;
                res.write("Unable to resolve sourcemap");
                res.end();
                return;
            }
            res.statusCode = 200;
            res.setHeader("Content-Type", "application/json");
            res.write(Buffer.from(JSON.stringify(originalStackFrame)));
            res.end();
            return;
        } else if (pathname === "/__nextjs_launch-editor") {
            if (!frame.file) {
                res.statusCode = 400;
                res.write("Bad Request");
                res.end();
                return;
            }
            const fileExists = await fs.access(frame.file, FS.F_OK).then(()=>true, ()=>false);
            if (!fileExists) {
                res.statusCode = 204;
                res.write("No Content");
                res.end();
                return;
            }
            try {
                var _frame_line, _frame_column;
                launchEditor(frame.file, (_frame_line = frame.line) != null ? _frame_line : 1, (_frame_column = frame.column) != null ? _frame_column : 1);
            } catch (err) {
                console.log("Failed to launch editor:", err);
                res.statusCode = 500;
                res.write("Internal Server Error");
                res.end();
                return;
            }
            res.statusCode = 204;
            res.end();
        }
    };
}

//# sourceMappingURL=middleware-turbopack.js.map